@@ -20,15 +20,17 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: cookie.h,v 1.22 2006-07-31 17:46:28 yangtse Exp $
+ * $Id: cookie.h,v 1.24 2006-08-30 16:17:06 giva Exp $
  ***************************************************************************/
 
 #include <stdio.h>
-#ifdef WIN32
+#if defined(WIN32)
 #include <time.h>
 #else
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#endif
 
 #include <curl/curl.h>
 
