@@ -19,7 +19,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: memdebug.c,v 1.51 2007-02-21 19:03:20 yangtse Exp $
+ * $Id: memdebug.c,v 1.53 2007-06-28 11:11:29 jehousley Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -73,6 +73,10 @@ void curl_memdebug(const char *logname)
       logfile = fopen(logname, "w");
     else
       logfile = stderr;
+#ifdef MEMDEBUG_LOG_SYNC
+    /* Flush the log file after every line so the log isn't lost in a crash */
+    setvbuf(logfile, (char *)NULL, _IOLBF, 0);
+#endif
   }
 }
 
@@ -276,6 +280,16 @@ FILE *curl_fopen(const char *file, const char *mode,
   return res;
 }
 
+FILE *curl_fdopen(int filedes, const char *mode,
+                  int line, const char *source)
+{
+  FILE *res=(fdopen)(filedes, mode);
+  if(logfile)
+    fprintf(logfile, "FILE %s:%d fdopen(\"%d\",\"%s\") = %p\n",
+            source, line, filedes, mode, res);
+  return res;
+}
+
 int curl_fclose(FILE *file, int line, const char *source)
 {
   int res;
