@@ -18,7 +18,7 @@
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
-* $Id: ssh.c,v 1.53 2007-06-20 11:30:35 jehousley Exp $
+* $Id: ssh.c,v 1.60 2007-07-05 12:48:34 jehousley Exp $
 ***************************************************************************/
 
 /* #define CURL_LIBSSH2_DEBUG */
@@ -384,7 +384,7 @@ static CURLcode ssh_statemach_act(struct connectdata *conn)
           sshc->rsa_pub = aprintf("%s", data->set.ssh_public_key);
         else if (home)
           sshc->rsa_pub = aprintf("%s/.ssh/id_dsa.pub", home);
-        
+
         if (sshc->rsa_pub == NULL) {
           curl_free(home);
           state(conn, SSH_SESSION_FREE);
@@ -1125,7 +1125,7 @@ CURLcode Curl_scp_do(struct connectdata *conn, bool *done)
 
   if (conn->data->set.upload) {
     if(conn->data->set.infilesize < 0) {
-      failf(conn->data, "SCP requries a known file size for upload");
+      failf(conn->data, "SCP requires a known file size for upload");
       return CURLE_UPLOAD_FAILED;
     }
     /*
@@ -1137,23 +1137,23 @@ CURLcode Curl_scp_do(struct connectdata *conn, bool *done)
 #if (LIBSSH2_APINO >= 200706012030)
     do {
       scp->ssh_channel = libssh2_scp_send_ex(scp->ssh_session, scp->path,
-                                             LIBSSH2_SFTP_S_IRUSR|
-                                             LIBSSH2_SFTP_S_IWUSR|
-                                             LIBSSH2_SFTP_S_IRGRP|
-                                             LIBSSH2_SFTP_S_IROTH,
+                                             conn->data->set.new_file_perms,
                                              conn->data->set.infilesize, 0, 0);
       if (!scp->ssh_channel &&
           (libssh2_session_last_errno(scp->ssh_session) !=
            LIBSSH2_ERROR_EAGAIN)) {
-        return CURLE_FAILED_INIT;
+        int err;
+        char *err_msg;
+
+        err = libssh2_session_error_to_CURLE(
+            libssh2_session_last_error(scp->ssh_session, &err_msg, NULL, 0));
+        failf(conn->data, "%s", err_msg);
+        return err;
       }
     } while (!scp->ssh_channel);
 #else /* !(LIBSSH2_APINO >= 200706012030) */
     scp->ssh_channel = libssh2_scp_send_ex(scp->ssh_session, scp->path,
-                                           LIBSSH2_SFTP_S_IRUSR|
-                                           LIBSSH2_SFTP_S_IWUSR|
-                                           LIBSSH2_SFTP_S_IRGRP|
-                                           LIBSSH2_SFTP_S_IROTH,
+                                           conn->data->set.new_file_perms,
                                            conn->data->set.infilesize, 0, 0);
     if (!scp->ssh_channel)
       return CURLE_FAILED_INIT;
@@ -1363,8 +1363,7 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
       sftp->sftp_handle =
         libssh2_sftp_open(sftp->sftp_session, sftp->path,
                         LIBSSH2_FXF_WRITE|LIBSSH2_FXF_CREAT|LIBSSH2_FXF_TRUNC,
-                        LIBSSH2_SFTP_S_IRUSR|LIBSSH2_SFTP_S_IWUSR|
-                        LIBSSH2_SFTP_S_IRGRP|LIBSSH2_SFTP_S_IROTH);
+                        data->set.new_file_perms);
       if (!sftp->sftp_handle &&
           (libssh2_session_last_errno(sftp->ssh_session) !=
            LIBSSH2_ERROR_EAGAIN)) {
@@ -1381,8 +1380,7 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
               sftp->sftp_handle = libssh2_sftp_open(sftp->sftp_session,
                           sftp->path,
                           LIBSSH2_FXF_WRITE|LIBSSH2_FXF_CREAT|LIBSSH2_FXF_TRUNC,
-                          LIBSSH2_SFTP_S_IRUSR|LIBSSH2_SFTP_S_IWUSR|
-                          LIBSSH2_SFTP_S_IRGRP|LIBSSH2_SFTP_S_IROTH);
+                          data->set.new_file_perms);
               if (!sftp->sftp_handle &&
                   (libssh2_session_last_errno(sftp->ssh_session) !=
                    LIBSSH2_ERROR_EAGAIN)) {
@@ -1406,8 +1404,7 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
     sftp->sftp_handle =
       libssh2_sftp_open(sftp->sftp_session, sftp->path,
                         LIBSSH2_FXF_WRITE|LIBSSH2_FXF_CREAT|LIBSSH2_FXF_TRUNC,
-                        LIBSSH2_SFTP_S_IRUSR|LIBSSH2_SFTP_S_IWUSR|
-                        LIBSSH2_SFTP_S_IRGRP|LIBSSH2_SFTP_S_IROTH);
+                        data->set.new_file_perms);
     if (!sftp->sftp_handle) {
       err = libssh2_sftp_last_error(sftp->sftp_session);
       if (((err == LIBSSH2_FX_NO_SUCH_FILE) ||
@@ -1420,8 +1417,7 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
         if (res == 0) {
           sftp->sftp_handle = libssh2_sftp_open(sftp->sftp_session, sftp->path,
                     LIBSSH2_FXF_WRITE|LIBSSH2_FXF_CREAT|LIBSSH2_FXF_TRUNC,
-                    LIBSSH2_SFTP_S_IRUSR|LIBSSH2_SFTP_S_IWUSR|
-                    LIBSSH2_SFTP_S_IRGRP|LIBSSH2_SFTP_S_IROTH);
+                    data->set.new_file_perms);
         }
       }
       if (!sftp->sftp_handle) {
@@ -1487,11 +1483,14 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
           filename[len] = '\0';
 
           if (data->set.ftp_list_only) {
-            if ((attrs.flags & LIBSSH2_SFTP_ATTR_PERMISSIONS) &&
-                ((attrs.permissions & LIBSSH2_SFTP_S_IFMT) ==
-                 LIBSSH2_SFTP_S_IFDIR)) {
-              infof(data, "%s\n", filename);
+            char *tmpLine;
+
+            tmpLine = aprintf("%s\n", filename);
+            if (tmpLine == NULL) {
+              return CURLE_OUT_OF_MEMORY;
             }
+            res = Curl_client_write(conn, CLIENTWRITE_BODY, tmpLine, 0);
+            Curl_safefree(tmpLine);
           }
           else {
             totalLen = 80 + len;
@@ -1633,8 +1632,7 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
       do {
         sftp->sftp_handle =
           libssh2_sftp_open(sftp->sftp_session, sftp->path, LIBSSH2_FXF_READ,
-                          LIBSSH2_SFTP_S_IRUSR|LIBSSH2_SFTP_S_IWUSR|
-                          LIBSSH2_SFTP_S_IRGRP|LIBSSH2_SFTP_S_IROTH);
+                          data->set.new_file_perms);
         if (!sftp->sftp_handle &&
             (libssh2_session_last_errno(sftp->ssh_session) !=
                                    LIBSSH2_ERROR_EAGAIN)) {
@@ -1647,8 +1645,7 @@ CURLcode Curl_sftp_do(struct connectdata *conn, bool *done)
 #else /* !(LIBSSH2_APINO >= 200706012030) */
       sftp->sftp_handle =
         libssh2_sftp_open(sftp->sftp_session, sftp->path, LIBSSH2_FXF_READ,
-                          LIBSSH2_SFTP_S_IRUSR|LIBSSH2_SFTP_S_IWUSR|
-                          LIBSSH2_SFTP_S_IRGRP|LIBSSH2_SFTP_S_IROTH);
+                          data->set.new_file_perms);
       if (!sftp->sftp_handle) {
         err = libssh2_sftp_last_error(sftp->sftp_session);
         failf(conn->data, "Could not open remote file for reading: %s",
@@ -2301,15 +2298,11 @@ static CURLcode sftp_create_dirs(struct connectdata *conn) {
       /* 'mode' - parameter is preliminary - default to 0644 */
 #if (LIBSSH2_APINO >= 200706012030)
       while ((rc = libssh2_sftp_mkdir(sftp->sftp_session, sftp->path,
-                               LIBSSH2_SFTP_S_IRWXU |
-                               LIBSSH2_SFTP_S_IRGRP | LIBSSH2_SFTP_S_IXGRP |
-                               LIBSSH2_SFTP_S_IROTH | LIBSSH2_SFTP_S_IXOTH)) ==
+                                      conn->data->set.new_directory_perms)) ==
              LIBSSH2_ERROR_EAGAIN);
 #else /* !(LIBSSH2_APINO >= 200706012030) */
       rc = libssh2_sftp_mkdir(sftp->sftp_session, sftp->path,
-                               LIBSSH2_SFTP_S_IRWXU |
-                               LIBSSH2_SFTP_S_IRGRP | LIBSSH2_SFTP_S_IXGRP |
-                               LIBSSH2_SFTP_S_IROTH | LIBSSH2_SFTP_S_IXOTH);
+                               conn->data->set.new_directory_perms);
 #endif /* !(LIBSSH2_APINO >= 200706012030) */
       *slash_pos = '/';
       ++slash_pos;
