@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: tftp.c,v 1.42 2007-04-11 00:25:41 danf Exp $
+ * $Id: tftp.c,v 1.48 2007-06-12 08:15:02 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -136,7 +136,6 @@ typedef struct tftp_state_data {
   time_t          max_time;
   unsigned short  block;
   struct Curl_sockaddr_storage   local_addr;
-  socklen_t       local_addrlen;
   struct Curl_sockaddr_storage   remote_addr;
   socklen_t       remote_addrlen;
   int             rbytes;
@@ -180,6 +179,10 @@ void tftp_set_timeouts(tftp_state_data_t *state)
     /* Average restart after 5 seconds */
     state->retry_max = timeout/5;
 
+    if(state->retry_max < 1)
+      /* avoid division by zero below */
+      state->retry_max = 1;
+
     /* Compute the re-start interval to suit the timeout */
     state->retry_time = timeout/state->retry_max;
     if(state->retry_time<1)
@@ -367,6 +370,7 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
                     state->remote_addrlen);
     if(sbytes < 0) {
       failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+      return CURLE_SEND_ERROR;
     }
 
     /* Check if completed (That is, a less than full packet is received) */
@@ -396,6 +400,7 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
       /* Check all sbytes were sent */
       if(sbytes<0) {
         failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+        return CURLE_SEND_ERROR;
       }
     }
     break;
@@ -406,10 +411,9 @@ static CURLcode tftp_rx(tftp_state_data_t *state, tftp_event_t event)
 
   default:
     failf(data, "%s\n", "tftp_rx: internal error");
-    break;
+    return CURLE_TFTP_ILLEGAL; /* not really the perfect return code for
+                                  this */
   }
-  Curl_pgrsSetDownloadCounter(data,
-                              (curl_off_t) state->block*TFTP_BLOCKSIZE);
   return CURLE_OK;
 }
 
@@ -426,6 +430,7 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
   int sbytes;
   int rblock;
   CURLcode res = CURLE_OK;
+  struct Curl_transfer_keeper *k = &data->reqdata.keep;
 
   switch(event) {
 
@@ -478,7 +483,11 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
     /* Check all sbytes were sent */
     if(sbytes<0) {
       failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+      return CURLE_SEND_ERROR;
     }
+    /* Update the progress meter */
+    k->writebytecount += state->sbytes;
+    Curl_pgrsSetUploadCounter(data, k->writebytecount);
     break;
 
   case TFTP_EVENT_TIMEOUT:
@@ -490,7 +499,8 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
     if(state->retries > state->retry_max) {
       state->error = TFTP_ERR_TIMEOUT;
       state->state = TFTP_STATE_FIN;
-    } else {
+    }
+    else {
       /* Re-send the data packet */
       sbytes = sendto(state->sockfd, (void *)&state->spacket,
                       4+state->sbytes, SEND_4TH_ARG,
@@ -499,7 +509,10 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
       /* Check all sbytes were sent */
       if(sbytes<0) {
         failf(data, "%s\n", Curl_strerror(state->conn, SOCKERRNO));
+        return CURLE_SEND_ERROR;
       }
+      /* since this was a re-send, we remain at the still byte position */
+      Curl_pgrsSetUploadCounter(data, k->writebytecount);
     }
     break;
 
@@ -512,9 +525,6 @@ static CURLcode tftp_tx(tftp_state_data_t *state, tftp_event_t event)
     break;
   }
 
-  /* Update the progress meter */
-  Curl_pgrsSetUploadCounter(data, (curl_off_t) state->block*TFTP_BLOCKSIZE);
-
   return res;
 }
 
@@ -659,6 +669,7 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
   struct Curl_sockaddr_storage fromaddr;
   socklen_t             fromlen;
   int                   check_time = 0;
+  struct Curl_transfer_keeper *k = &data->reqdata.keep;
 
   *done = TRUE;
 
@@ -675,13 +686,18 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
     state = (tftp_state_data_t *)conn->data->reqdata.proto.tftp;
   }
 
+  code = Curl_readwrite_init(conn);
+  if(code)
+    return code;
+
   /* Run the TFTP State Machine */
   for(code=tftp_state_machine(state, TFTP_EVENT_INIT);
       (state->state != TFTP_STATE_FIN) && (code == CURLE_OK);
       code=tftp_state_machine(state, event) ) {
 
     /* Wait until ready to read or timeout occurs */
-    rc=Curl_socket_ready(state->sockfd, CURL_SOCKET_BAD, state->retry_time * 1000);
+    rc=Curl_socket_ready(state->sockfd, CURL_SOCKET_BAD,
+                         state->retry_time * 1000);
 
     if(rc == -1) {
       /* bail out */
@@ -730,6 +746,8 @@ CURLcode Curl_tftp(struct connectdata *conn, bool *done)
                                      state->rbytes-4);
             if(code)
               return code;
+            k->bytecount += state->rbytes-4;
+            Curl_pgrsSetDownloadCounter(data, (curl_off_t) k->bytecount);
           }
           break;
         case TFTP_EVENT_ERROR:
