@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: transfer.c,v 1.355 2007-06-22 20:24:11 bagder Exp $
+ * $Id: transfer.c,v 1.357 2007-06-25 14:17:52 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -574,14 +574,16 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                 k->header = FALSE; /* no more header to parse! */
 
                 if((k->size == -1) && !conn->bits.chunk && !conn->bits.close &&
-                   (k->httpversion >= 11) )
+                   (k->httpversion >= 11) ) {
                   /* On HTTP 1.1, when connection is not to get closed, but no
                      Content-Length nor Content-Encoding chunked have been
-                     received, there is no body in this response. We don't set
-                     stop_reading TRUE since that would also prevent necessary
-                     authentication actions to take place. */
-                  conn->bits.no_body = TRUE;
-
+                     received, according to RFC2616 section 4.4 point 5, we
+                     assume that the server will close the connection to
+                     signal the end of the document. */
+                  infof(data, "no chunk, no close, no size. Assume close to "
+                        "signal end\n");
+                  conn->bits.close = TRUE;
+                }
               }
 
               if (417 == k->httpcode) {
@@ -800,11 +802,13 @@ CURLcode Curl_readwrite(struct connectdata *conn,
                   }
                 }
 
-                if(k->httpversion == 10)
+                if(k->httpversion == 10) {
                   /* Default action for HTTP/1.0 must be to close, unless
                      we get one of those fancy headers that tell us the
                      server keeps it open for us! */
+                  infof(data, "HTTP 1.0, assume close after body\n");
                   conn->bits.close = TRUE;
+                }
 
                 switch(k->httpcode) {
                 case 204:
@@ -1720,8 +1724,6 @@ CURLcode Curl_readwrite_init(struct connectdata *conn)
  */
 void Curl_pre_readwrite(struct connectdata *conn)
 {
-  DEBUGF(infof(conn->data, "Pre readwrite setting chunky header "
-               "values to default\n"));
   conn->bits.chunk=FALSE;
   conn->bits.trailerHdrPresent=FALSE;
 }
