@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: ftpserver.pl,v 1.79 2007-03-01 16:42:02 yangtse Exp $
+# $Id: ftpserver.pl,v 1.81 2007-04-24 21:30:39 danf Exp $
 ###########################################################################
 
 # This is the FTP server designed for the curl test suite.
@@ -139,16 +139,18 @@ sub sysread_or_die {
         logmsg "Failed to read input\n";
         logmsg "Error: ftp$ftpdnum$ext sysread error: $!\n";
         kill(9, $sfpid);
-        die "Died in sysread_or_die() when called from $fcaller " .
-            "at line $lcaller. ftp$ftpdnum$ext sysread error: $!\n";
+        waitpid($sfpid, 0);
+        die "Died in sysread_or_die() at $fcaller " .
+            "line $lcaller. ftp$ftpdnum$ext sysread error: $!\n";
     }
     elsif($result == 0) {
         ($fcaller, $lcaller) = (caller)[1,2];
         logmsg "Failed to read input\n";
         logmsg "Error: ftp$ftpdnum$ext read zero\n";
         kill(9, $sfpid);
-        die "Died in sysread_or_die() when called from $fcaller " .
-            "at line $lcaller. ftp$ftpdnum$ext read zero\n";
+        waitpid($sfpid, 0);
+        die "Died in sysread_or_die() at $fcaller " .
+            "line $lcaller. ftp$ftpdnum$ext read zero\n";
     }
 
     return $result;
@@ -167,6 +169,7 @@ sub startsf {
     if($pong !~ /^PONG/) {
         logmsg "Failed sockfilt command: $cmd\n";
         kill(9, $sfpid);
+        waitpid($sfpid, 0);
         die "Failed to start sockfilt!";
     }
 }
@@ -569,6 +572,7 @@ sub PASV_command {
 
     if($pong !~ /^PONG/) {
         kill(9, $slavepid);
+        waitpid($slavepid, 0);
         sendcontrol "500 no free ports!\r\n";
         logmsg "failed to run sockfilt for data connection\n";
         return 0;
@@ -697,6 +701,7 @@ sub PORT_command {
     if($pong !~ /^PONG/) {
         logmsg "Failed sockfilt for data connection\n";
         kill(9, $slavepid);
+        waitpid($slavepid, 0);
     }
 
     logmsg "====> Client DATA connect to port $port\n";
@@ -712,6 +717,12 @@ sub customize {
 
     $nosave = 0; # default is to save as normal
     $controldelay = 0; # default is no delaying the responses
+    $retrweirdo = 0;
+    $retrnosize = 0;
+    $pasvbadip = 0;
+    $nosave = 0;
+    %customcount = ();
+    %delayreply = ();
 
     open(CUSTOM, "<log/ftpserver.cmd") ||
         return 1;
@@ -787,6 +798,7 @@ while(1) {
     $| = 1;
 
     kill(9, $slavepid) if($slavepid);
+    waitpid($slavepid, 0) if($slavepid);
     $slavepid=0;
         
     &customize(); # read test control instructions
