@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: runtests.pl,v 1.234 2007-04-10 00:37:56 danf Exp $
+# $Id: runtests.pl,v 1.247 2007-06-11 17:53:10 danf Exp $
 ###########################################################################
 # These should be the only variables that might be needed to get edited:
 
@@ -49,6 +49,7 @@ my $FTP6PORT; # FTP IPv6 server port
 my $TFTPPORT; # TFTP
 my $TFTP6PORT; # TFTP
 my $SSHPORT; # SCP/SFTP
+my $SOCKSPORT; # SOCKS4/5 port
 
 my $CURL="../src/curl"; # what curl executable to run on the tests
 my $DBGCURL=$CURL; #"../src/.libs/curl";  # alternative for debugging
@@ -81,6 +82,7 @@ my $FTPSPIDFILE=".ftps.pid";
 my $TFTPPIDFILE=".tftpd.pid";
 my $TFTP6PIDFILE=".tftp6.pid";
 my $SSHPIDFILE=".ssh.pid";
+my $SOCKSPIDFILE=".socks.pid";
 
 # invoke perl like this:
 my $perl="perl -I$srcdir";
@@ -256,7 +258,7 @@ sub checkdied {
 # Return the pids (yes plural) of the new child process to the parent.
 #
 sub startnew {
-    my ($cmd, $pidfile)=@_;
+    my ($cmd, $pidfile,$fake)=@_;
 
     logmsg "startnew: $cmd\n" if ($verbose);
 
@@ -271,20 +273,29 @@ sub startnew {
     if(0 == $child) {
         # Here we are the child. Run the given command.
 
-        # Calling exec() within a pseudo-process actually spawns the requested
-        # executable in a separate process and waits for it to complete before
-        # exiting with the same exit status as that process.  This means that
-        # the process ID reported within the running executable will be
-        # different from what the earlier Perl fork() might have returned.
+        # Put an "exec" in front of the command so that the child process
+        # keeps this child's process ID.
+        exec("exec $cmd") || die "Can't exec() $cmd: $!";
 
         # exec() should never return back here to this process. We protect
-        # ourselfs calling die() just in case something goes really bad.
-
-        exec($cmd) || die "Can't exec() $cmd: $!";
-
+        # ourselves by calling die() just in case something goes really bad.
         die "error: exec() has returned";
     }
 
+
+    # Ugly hack but ssh doesn't support pid files
+    if ($fake) {
+        logmsg "$pidfile faked with pid=$child\n" if($verbose);
+        open(OUT, ">$pidfile");
+        print OUT $child . "\n";
+        close(OUT);
+	# could/should do a while connect fails sleep a bit and loop
+	sleep 5;
+        if (checkdied($child)) {
+            logmsg "startnew: Warning: child process has failed to start\n" if($verbose);
+            return (-1,-1);
+        }
+    }
     my $count=12;
     while($count--) {
         if(-f $pidfile) {
@@ -310,6 +321,9 @@ sub startnew {
         sleep(1);
     }
 
+    # Return two PIDs, the one for the child process we spawned and the one
+    # reported by the server itself (in case it forked again on its own).
+    # Both (potentially) need to be killed at the end of the test.
     return ($child, $pid2);
 }
 
@@ -437,7 +451,7 @@ sub torture {
 sub stopserver {
     my ($pid) = @_;
 
-    if(not defined $pid) {
+    if(not defined $pid || $pid <= 0) {
         return; # whad'da'ya wanna'da with no pid ?
     }
 
@@ -574,7 +588,18 @@ sub verifyftp {
 
 sub verifyssh {
     my ($proto, $ip, $port) = @_;
-    open(FILE, "<" . $SSHPIDFILE);
+    open(FILE, "<$SSHPIDFILE");
+    my $pid=0+<FILE>;
+    close(FILE);
+    return $pid;
+}
+
+#######################################################################
+# STUB for verifying socks
+
+sub verifysocks {
+    my ($proto, $ip, $port) = @_;
+    open(FILE, "<$SOCKSPIDFILE");
     my $pid=0+<FILE>;
     close(FILE);
     return $pid;
@@ -582,7 +607,9 @@ sub verifyssh {
 
 #######################################################################
 # Verify that the server that runs on $ip, $port is our server.
-# Retry during 5 seconds before giving up.
+# Retry over several seconds before giving up.  The ssh server in
+# particular can take a long time to start if it needs to generate
+# keys on a slow or loaded host.
 #
 
 my %protofunc = ('http' => \&verifyhttp,
@@ -590,12 +617,13 @@ my %protofunc = ('http' => \&verifyhttp,
                  'ftp' => \&verifyftp,
                  'ftps' => \&verifyftp,
                  'tftp' => \&verifyftp,
-                 'ssh' => \&verifyssh);
+                 'ssh' => \&verifyssh,
+                 'socks' => \&verifysocks);
 
 sub verifyserver {
     my ($proto, $ip, $port) = @_;
 
-    my $count = 5; # try for this many seconds
+    my $count = 30; # try for this many seconds
     my $pid;
 
     while($count--) {
@@ -648,9 +676,9 @@ sub runhttpserver {
 
     my $cmd="$perl $srcdir/httpserver.pl -p $pidfile $fork$flag $port $ipv6";
     my ($httppid, $pid2) =
-        startnew($cmd, $pidfile); # start the server in a new process
+        startnew($cmd, $pidfile,0); # start the server in a new process
 
-    if(!kill(0, $httppid)) {
+    if($httppid <= 0 || !kill(0, $httppid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTP server\n";
         stopservers($verbose);
@@ -702,9 +730,9 @@ sub runhttpsserver {
     my $flag=$debugprotocol?"-v ":"";
     my $cmd="$perl $srcdir/httpsserver.pl $flag -p https -s \"$stunnel\" -d $srcdir -r $HTTPPORT $HTTPSPORT";
 
-    my ($httpspid, $pid2) = startnew($cmd, $HTTPSPIDFILE);
+    my ($httpspid, $pid2) = startnew($cmd, $HTTPSPIDFILE,0);
 
-    if(!kill(0, $httpspid)) {
+    if($httpspid <= 0 || !kill(0, $httpspid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the HTTPS server\n";
         stopservers($verbose);
@@ -768,9 +796,9 @@ sub runftpserver {
 
     unlink($pidfile);
 
-    my ($ftppid, $pid2) = startnew($cmd, $pidfile);
+    my ($ftppid, $pid2) = startnew($cmd, $pidfile,0);
 
-    if(!$ftppid || !kill(0, $ftppid)) {
+    if($ftppid <= 0 || !kill(0, $ftppid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the FTP$id$nameext server\n";
         return -1;
@@ -821,9 +849,9 @@ sub runftpsserver {
     my $flag=$debugprotocol?"-v ":"";
     my $cmd="$perl $srcdir/httpsserver.pl $flag -p ftps -s \"$stunnel\" -d $srcdir -r $FTPPORT $FTPSPORT";
 
-    my ($ftpspid, $pid2) = startnew($cmd, $FTPSPIDFILE);
+    my ($ftpspid, $pid2) = startnew($cmd, $FTPSPIDFILE,0);
 
-    if(!kill(0, $ftpspid)) {
+    if($ftpspid <= 0 || !kill(0, $ftpspid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the FTPS server\n";
         stopservers($verbose);
@@ -887,9 +915,9 @@ sub runtftpserver {
 
     unlink($pidfile);
 
-    my ($tftppid, $pid2) = startnew($cmd, $pidfile);
+    my ($tftppid, $pid2) = startnew($cmd, $pidfile,0);
 
-    if(!$tftppid || !kill(0, $tftppid)) {
+    if($tftppid <= 0 || !kill(0, $tftppid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the FTP$id$nameext server\n";
         return -1;
@@ -930,9 +958,9 @@ sub runsshserver {
     my $flag=$debugprotocol?"-v ":"";
     my $cmd="$perl $srcdir/sshserver.pl $flag-u $USER -d $srcdir $port";
     my ($sshpid, $pid2) =
-        startnew($cmd, $pidfile); # start the server in a new process
+        startnew($cmd, $pidfile,0); # start the server in a new process
 
-    if(!$sshpid || !kill(0, $sshpid)) {
+    if($sshpid <= 0 || !kill(0, $sshpid)) {
         # it is NOT alive
         logmsg "RUN: failed to start the SSH server\n";
         # failed to talk to it properly. Kill the server and return failure
@@ -951,6 +979,40 @@ sub runsshserver {
     return ($pid2, $sshpid);
 }
 
+#######################################################################
+# Start the socks server
+#
+sub runsocksserver {
+    my ($id, $verbose, $ipv6) = @_;
+    my $ip=$HOSTIP;
+    my $port = $SOCKSPORT;
+    my $pidfile = $SOCKSPIDFILE;
+
+    my $flag=$debugprotocol?"-v ":"";
+    my $cmd="ssh -D ${HOSTIP}:$SOCKSPORT -N -F curl_ssh_config ${USER}\@${HOSTIP} -p ${SSHPORT} -vv >log/ssh.log 2>&1";
+    my ($sshpid, $pid2) =
+        startnew($cmd, $pidfile,1); # start the server in a new process
+
+    if($sshpid <= 0 || !kill(0, $sshpid)) {
+        # it is NOT alive
+        logmsg "RUN: failed to start the SOCKS server\n";
+        # failed to talk to it properly. Kill the server and return failure
+        stopserver("$sshpid $pid2");
+        return (0,0);
+    }
+
+    # Ugly hack but ssh doesn't support pid files
+    if (!verifyserver('socks',$ip,$port)) {
+        logmsg "RUN: SOCKS server failed verification\n";
+        return (0,0);
+    }
+    if($verbose) {
+        logmsg "RUN: SOCKS server is now running PID $sshpid\n";
+    }
+
+    return ($pid2, $sshpid);
+}
+
 #######################################################################
 # Remove all files in the specified directory
 #
@@ -1045,7 +1107,7 @@ sub checksystem {
     $versretval = system($versioncmd);
     $versnoexec = $!;
 
-    open(VERSOUT, $curlverout);
+    open(VERSOUT, "<$curlverout");
     @version = <VERSOUT>;
     close(VERSOUT);
 
@@ -1267,6 +1329,7 @@ sub checksystem {
         logmsg sprintf("* TFTP IPv6 port: %d\n", $TFTP6PORT);
     }
     logmsg sprintf("* SCP/SFTP port:  %d\n", $SSHPORT);
+    logmsg sprintf("* SOCKS port:     %d\n", $SOCKSPORT);
 
     if($ssl_version) {
         logmsg sprintf("* SSL library:    %s\n", $ssllib);
@@ -1298,6 +1361,7 @@ sub subVariables {
   $$thing =~ s/%TFTPPORT/$TFTPPORT/g;
   $$thing =~ s/%TFTP6PORT/$TFTP6PORT/g;
   $$thing =~ s/%SSHPORT/$SSHPORT/g;
+  $$thing =~ s/%SOCKSPORT/$SOCKSPORT/g;
   $$thing =~ s/%CURL/$CURL/g;
   $$thing =~ s/%USER/$USER/g;
 
@@ -1412,6 +1476,9 @@ sub singletest {
                 next;
             }
         }
+        elsif($f eq "socks") {
+            next;
+        }
         # See if this "feature" is in the list of supported protocols
         elsif (grep /^$f$/, @protocols) {
             next;
@@ -1567,10 +1634,10 @@ sub singletest {
         my $fileContent = join('', @inputfile);
         subVariables \$fileContent;
 #        logmsg "DEBUG: writing file " . $filename . "\n";
-        open OUTFILE, ">$filename";
+        open(OUTFILE, ">$filename");
         binmode OUTFILE; # for crapage systems, use binary
         print OUTFILE $fileContent;
-        close OUTFILE;
+        close(OUTFILE);
     }
 
     my %cmdhash = getpartattr("client", "command");
@@ -1633,6 +1700,7 @@ sub singletest {
     my $dumped_core;
     my $cmdres;
 
+    # Apr 2007: precommand isn't being used and could be removed
     my @precommand= getpart("client", "precommand");
     if($precommand[0]) {
         # this is pure perl to eval!
@@ -1654,7 +1722,7 @@ sub singletest {
     }
     # run the command line we built
     if ($torture) {
-        return torture($CMDLINE,
+        $cmdres = torture($CMDLINE,
                        "$gdb --directory libtest $DBGCURL -x log/gdbcmd");
     }
     elsif($gdbthis) {
@@ -1692,6 +1760,20 @@ sub singletest {
         }
     }
 
+    # run the postcheck command
+    my @postcheck= getpart("client", "postcheck");
+    $cmd = $postcheck[0];
+    chomp $cmd;
+    subVariables \$cmd;
+    if($cmd) {
+	my $rc = system("$cmd");
+	if($rc != 0) {
+	    logmsg "postcheck failure\n";
+	    return 1;
+	}
+	logmsg "postchecked $cmd\n" if($verbose);
+    }
+
     # remove the special FTP command file after each test!
     unlink($FTPDCMD);
 
@@ -1700,6 +1782,14 @@ sub singletest {
         $ENV{$e}=""; # clean up
     }
 
+    # Skip all the verification on torture tests
+    if ($torture) {
+	if(!$cmdres && !$keepoutfiles) {
+	    cleardir($LOGDIR);
+	}
+        return $cmdres;
+    }
+
     my @err = getpart("verify", "errorcode");
     my $errorcode = $err[0] || "0";
     my $ok="";
@@ -1943,7 +2033,7 @@ sub singletest {
             opendir(DIR, "log") ||
                 return 0; # can't open log dir
             my @files = readdir(DIR);
-            closedir DIR;
+            closedir(DIR);
             my $f;
             my $l;
             foreach $f (@files) {
@@ -2160,7 +2250,7 @@ sub startservers {
                 $run{'tftp-ipv6'}="$pid $pid2";
             }
         }
-        elsif($what eq "sftp" || $what eq "scp") {
+        elsif($what eq "sftp" || $what eq "scp" || $what eq "socks4" || $what eq "socks5" ) {
             if(!$run{'ssh'}) {
                 ($pid, $pid2) = runsshserver("", $verbose);
                 if($pid <= 0) {
@@ -2169,6 +2259,29 @@ sub startservers {
                 printf ("* pid ssh => %d %d\n", $pid, $pid2) if($verbose);
                 $run{'ssh'}="$pid $pid2";
             }
+	    if ($what eq "socks4" || $what eq "socks5") {
+                if (!checkcmd("ssh")) {
+                   return "failed to find SSH client for socks support";
+		}
+            	if(!$run{'socks'}) {
+		    my $sshversion=`ssh -V 2>&1`;
+                    if ($sshversion =~ /SSH_(\d+)\.(\d+)/i) {
+                       if ($1*10+$2 < 37) {
+		       # need 3.7 for socks5 - http://www.openssh.com/txt/release-3.7
+                           return "ssh version ($1.$2) insufficient; need at least 3.7";
+		       }
+                    } else {
+                       return "Unsupported ssh client\n";
+                    }
+
+                    ($pid, $pid2) = runsocksserver("", $verbose);
+                    if($pid <= 0) {
+                        return "failed starting socks server";
+                    }
+                    printf ("* pid socks => %d %d\n", $pid, $pid2) if($verbose);
+                    $run{'socks'}="$pid $pid2";
+		}
+	    }
         }
         elsif($what eq "none") {
             logmsg "* starts no server\n" if ($verbose);
@@ -2211,7 +2324,9 @@ sub serverfortest {
     my $proto = lc($what[0]);
     chomp $proto;
     if (! grep /^$proto$/, @protocols) {
-        return "curl lacks $proto support";
+        if (substr($proto,0,5) ne "socks") {
+		return "curl lacks any $proto support";
+        }
     }
 
     return &startservers(@what);
@@ -2224,11 +2339,18 @@ sub serverfortest {
 my $number=0;
 my $fromnum=-1;
 my @testthis;
+my %disabled;
 do {
     if ($ARGV[0] eq "-v") {
         # verbose output
         $verbose=1;
     }
+    elsif($ARGV[0] =~ /^-b(.*)/) {
+        my $portno=$1;
+        if($portno =~ s/(\d+)$//) {
+            $base = int $1;
+        }
+    }
     elsif ($ARGV[0] eq "-c") {
         # use this path to curl instead of default
         $DBGCURL=$CURL=$ARGV[1];
@@ -2285,19 +2407,22 @@ do {
     elsif($ARGV[0] eq "-h") {
         # show help text
         print <<EOHELP
-Usage: runtests.pl [options]
+Usage: runtests.pl [options] [test number(s)]
   -a       continue even if a test fails
+  -bN      use base port number N for test servers (default $base)
+  -c path  use this curl executable
   -d       display server debug info
   -g       run the test case with gdb
   -h       this help text
   -k       keep stdout and stderr files present after tests
   -l       list all test case names/descriptions
-  -n       No valgrind
-  -p       Print log file contents when a test fails
+  -n       no valgrind
+  -p       print log file contents when a test fails
   -s       short output
-  -t       torture
+  -t[N]    torture (simulate memory alloc failures); N means fail Nth alloc
   -v       verbose output
   [num]    like "5 6 9" or " 5 to 22 " to run those tests only
+  [!num]   like "!5 !6 !9" to disable those tests
 EOHELP
     ;
         exit;
@@ -2317,6 +2442,10 @@ EOHELP
     elsif($ARGV[0] =~ /^to$/i) {
         $fromnum = $number+1;
     }
+    elsif($ARGV[0] =~ /^!(\d+)/) {
+        $fromnum = -1;
+        $disabled{$1}=$1;
+    }
 } while(shift @ARGV);
 
 if($testthis[0] ne "") {
@@ -2357,6 +2486,7 @@ $FTP6PORT =  $base + 6; # FTP IPv6 port
 $TFTPPORT =  $base + 7; # TFTP (UDP) port
 $TFTP6PORT =  $base + 8; # TFTP IPv6 (UDP) port
 $SSHPORT =   $base + 9; # SSH (SCP/SFTP) port
+$SOCKSPORT =   $base + 10; # SOCKS port
 
 #######################################################################
 # clear and create logging directory:
@@ -2381,19 +2511,19 @@ if ( $TESTCASES eq "all") {
     # Get all commands and find out their test numbers
     opendir(DIR, $TESTDIR) || die "can't opendir $TESTDIR: $!";
     my @cmds = grep { /^test([0-9]+)$/ && -f "$TESTDIR/$_" } readdir(DIR);
-    closedir DIR;
+    closedir(DIR);
 
-    my %dis;
-    open(D, "$TESTDIR/DISABLED");
+    open(D, "<$TESTDIR/DISABLED");
     while(<D>) {
         if(/^ *\#/) {
             # allow comments
             next;
         }
         if($_ =~ /(\d+)/) {
-            $dis{$1}=$1; # disable this test number
+            $disabled{$1}=$1; # disable this test number
         }
     }
+    close(D);
 
     $TESTCASES=""; # start with no test cases
 
@@ -2403,9 +2533,9 @@ if ( $TESTCASES eq "all") {
     }
     # the the numbers from low to high
     foreach my $n (sort { $a <=> $b } @cmds) {
-        if($dis{$n}) {
+        if($disabled{$n}) {
             # skip disabled test cases
-            my $why = "mentioned in DISABLED";
+            my $why = "configured as DISABLED";
             $skipped++;
             $skipped{$why}++;
             $teststat[$n]=$why; # store reason for this test case
@@ -2423,11 +2553,16 @@ open(CMDLOG, ">$CURLLOG") ||
 
 #######################################################################
 
+# Display the contents of the given file.  Line endings are canonicalized
+# and excessively long files are truncated
 sub displaylogcontent {
     my ($file)=@_;
-    if(open(my $SINGLE, "<$file")) {
+    if(open(SINGLE, "<$file")) {
         my $lfcount;
-        while(my $string = <$SINGLE>) {
+        my $linecount = 0;
+        my $truncate;
+        my @tail;
+        while(my $string = <SINGLE>) {
             $string =~ s/\r\n/\n/g;
             $string =~ s/[\r\f\032]/\n/g;
             $string .= "\n" unless ($string =~ /\n$/);
@@ -2435,25 +2570,41 @@ sub displaylogcontent {
             if($lfcount == 1) {
                 $string =~ s/\n//;
                 $string =~ s/\s*\!$//;
-                logmsg " $string\n";
+                $linecount++;
+                if ($truncate) {
+                    push @tail, " $string\n";
+                } else {
+                    logmsg " $string\n";
+                }
             }
             else {
                 for my $line (split("\n", $string)) {
                     $line =~ s/\s*\!$//;
-                    logmsg " $line\n";
+                    $linecount++;
+                    if ($truncate) {
+                        push @tail, " $line\n";
+                    } else {
+                        logmsg " $line\n";
+		    }
                 }
             }
+            $truncate = $linecount > 1000;
         }
-        close($SINGLE);
+        if (@tail) {
+            logmsg "=== File too long: lines here were removed\n";
+            # This won't work properly if time stamps are enabled in logmsg
+            logmsg join('',@tail[$#tail-200..$#tail]);
+        }
+        close(SINGLE);
     }
 }
 
 sub displaylogs {
     my ($testnum)=@_;
-    opendir(my $DIR, "$LOGDIR") ||
+    opendir(DIR, "$LOGDIR") ||
         die "can't open dir: $!";
-    my @logs = readdir($DIR);
-    closedir($DIR);
+    my @logs = readdir(DIR);
+    closedir(DIR);
 
     logmsg "== Contents of files in the log/ dir after test $testnum\n";
     foreach my $log (sort @logs) {
@@ -2484,6 +2635,9 @@ sub displaylogs {
         if(($log =~ /^test\d+\.txt/) && ($log !~ /^test$testnum\.txt/)) {
             next; # skip testNnn.txt of other tests
         }
+        if(($log =~ /^file\d+\.txt/) && ($log !~ /^file$testnum\.txt/)) {
+            next; # skip fileNnn.txt of other tests
+        }
         logmsg "=== Start of file $log\n";
         displaylogcontent("$LOGDIR/$log");
         logmsg "=== End of file $log\n";
